###############################
### The epidemiological ODE ###
###############################

vector_epi_model          <- function(t, y, parms) {
  
  derivs = with(
    c(
      as.list(y)
    , as.list(parms)
      )
    
  , c(
    
    ## dS_h / dt
    mu_h * N_h - 
      (B * beta_mh * (I_m / N_h) + mu_h) * S_h   
    
    ## dI_h / dt
  , B * beta_mh * (I_m / N_h) * S_h - 
      (eta_h + mu_h) * I_h          
    
    ## dR_h / dt
  , eta_h * I_h -
      mu_h * R_h       
    
    ## dN_h / dt 
  , mu_h * N_h - 
      mu_h * S_h - mu_h * I_h - 
      mu_h * R_h                  
    
    ## dA_m / dt
  , phi * (1 - (A_m / (k * N_h))) * (S_m + I_m) - 
      (eta_a + mu_a) * A_m  
  
    ## dS_m / dt  
  , eta_a * A_m - 
      (B * beta_hm * (I_h / N_h) + mu_m) * S_m           
  
    ## dI_m / dt
  , B * beta_hm * (I_h / N_h) * S_m - 
      mu_m * I_m   
    
    )
  )
  
  list(derivs, NULL)
  
}

vector_epi_model_expanded <- function(t, y, parms) {
  
  derivs = with(
    c(
      as.list(y)
    , as.list(parms)
      )
    
  , c(
    
    ## dS_h / dt (Susceptible hosts)
    mu_h * N_h - 
      (B * beta_mh * (I_m / N_h) + mu_h) * S_h +
      delta_h * R_h
    
    ## dE_h / dt (Exposed hosts)  
  , B * beta_mh * (I_m / N_h) * S_h * (omega_Eh) - 
      (gamma_h + mu_h) * E_h           
    
    ## dI_h / dt (Infected hosts)
  , B * beta_mh * (I_m / N_h) * S_h * (1 - omega_Eh) * (1 - omega_Ah) +
      gamma_h * E_h * (1 - omega_Ah) - 
      (eta_h + alpha_h + mu_h) * I_h      
  
    ## dI_Ah / dt (Infected but asymptomatic hosts)
  , B * beta_mh * (I_m / N_h) * S_h * (1 - omega_Eh) * (omega_Ah) +
      gamma_h * E_h * (omega_Ah) - 
      (eta_h + mu_h) * I_Ah   
    
    ## dR_h / dt (Recovered hosts)
  , eta_h * (I_h + I_Ah) -
      (delta_h + mu_h) * R_h
    
    ## dN_h / dt (Total hosts)
  , mu_h * N_h - 
      mu_h * (S_h + E_h + I_h + I_Ah + R_h) - 
      alpha_h * I_h                 
    
    ## dA_m / dt (Aquatic mosquitoes)
  , phi * (1 - (A_m / (k * N_h))) * (S_m + I_m) - 
      (eta_a + mu_a) * A_m  
  
    ## dS_m / dt (Susceptible mosquitoes) 
  , eta_a * A_m - 
      (B * beta_hm * ((I_h / N_h) + (I_Ah / N_h) * psi_h) + mu_m) * S_m
    
    ## dE_m / dt (Exposed mosquitoes) 
  , B * beta_hm * ((I_h / N_h) + (I_Ah / N_h) * psi_h) * (omega_Em) * S_m - 
      gamma_m * E_m -
      mu_m * E_m   
    
    ## dI_m / dt (Infected mosquitoes)
  , B * beta_hm * ((I_h / N_h) + (I_Ah / N_h) * psi_h) * (1 - omega_Em) * S_m +
      gamma_m * E_m -
      mu_m * I_m   
    
    )
  )
  
  list(derivs, NULL)
  
}
